@@ -10,8 +10,8 @@ namespace glz
    template <class T>
    concept is_value_proxy = requires { T::glaze_value_proxy; };
 
-   template <is_value_proxy T>
-   struct from<JSON, T>
+   template <uint32_t Format, is_value_proxy T>
+   struct from<Format, T>
    {
       template <auto Opts>
       static void op(auto&& value, is_context auto&& ctx, auto&& it, auto&& end)
